@@ -16,6 +16,7 @@ typedef enum {
     BD_FS_ERROR_NOT_MOUNTED,
     BD_FS_ERROR_AUTH, // keep this entry last (XXX?)
     BD_FS_ERROR_TECH_UNAVAIL,
+    BD_FS_ERROR_UNKNOWN_FS = 11,
 } BDFsError;
 
 /* XXX: where the file systems start at the enum of technologies */
@@ -54,9 +55,9 @@ typedef enum {
  * close()      - clean after the plugin at the end or if no longer used
  *
  */
-gboolean bd_fs_check_deps ();
-gboolean bd_fs_init ();
-void bd_fs_close ();
+gboolean bd_fs_check_deps (void);
+gboolean bd_fs_init (void);
+void bd_fs_close (void);
 
 gboolean bd_fs_is_tech_avail (BDFSTech tech, guint64 mode, GError **error);
 
