@@ -812,6 +812,7 @@ gboolean bd_kbd_bcache_create (const gchar *backing_device, const gchar *cache_d
         if (!present) {
             g_set_error (error, BD_KBD_ERROR, BD_KBD_ERROR_BCACHE_NOEXIST,
                          "Failed to locate uuid symlink '%s'", device_uuid[i]);
+            bd_utils_report_finished (progress_id, (*error)->message);
             return FALSE;
         }
      }
@@ -843,6 +844,8 @@ gboolean bd_kbd_bcache_create (const gchar *backing_device, const gchar *cache_d
     }
     if (!dev_name) {
         globfree (&globbuf);
+        g_set_error (error, BD_KBD_ERROR, BD_KBD_ERROR_BCACHE_SETUP_FAIL,
+                     "Failed to determine bcache device name");
         bd_utils_report_finished (progress_id, (*error)->message);
         return FALSE;
     }
@@ -959,6 +962,7 @@ gboolean bd_kbd_bcache_detach (const gchar *bcache_device, gchar **c_set_uuid, G
     path = g_strdup_printf ("/sys/block/%s/bcache/detach", bcache_device);
     success = bd_utils_echo_str_to_file (uuid, path, error);
     if (!success) {
+        g_clear_error (error);
         g_set_error (error, BD_KBD_ERROR, BD_KBD_ERROR_BCACHE_DETACH_FAIL,
                      "Failed to detach '%s' from '%s'", uuid, bcache_device);
         g_free (link);
@@ -1035,6 +1039,7 @@ gboolean bd_kbd_bcache_destroy (const gchar *bcache_device, GError **error) {
 
     if (c_set_uuid) {
         path = g_strdup_printf ("/sys/fs/bcache/%s/stop", c_set_uuid);
+        g_free (c_set_uuid);
         success = bd_utils_echo_str_to_file ("1", path, error);
         g_free (path);
         if (!success) {
@@ -1254,6 +1259,7 @@ static gboolean get_cache_size_used (const gchar *cache_dev_sys, guint64 *size,
     g_io_channel_unref (file);
 
     if (!found) {
+        g_free (line);
         g_set_error (error, BD_KBD_ERROR, BD_KBD_ERROR_BCACHE_INVAL,
                      "Failed to get cache usage data");
         return FALSE;
